/*
 * Compiler.java
 *
 * Created on 6  2002 ., 11:15
 */

package com.fn.spcave.compiler;


import java.io.File;


/**
 *
 * @author  asso
 */
public class Compiler {

    private File inputDir;
    private File outputDir;
    private Locale locale;
    private Project project;
    
    /** Creates a new instance of Compiler */
    public Compiler(File inputDir, File outputDir, Locale locale, Project project) {
        this.inputDir = inputDir;
        this.outputDir = outputDir;
        this.locale = locale;
        this.project = project;
    }
    
    public void run() throws ScException {
        System.out.println("Generation started");
        if (! outputDir.exists())
            outputDir.mkdirs();
        processDirectory(inputDir, "");
        System.out.println("Generation finished");
    }
    
    private class FileName {
        public String fullName;
        public String name;
        public String localeName;
        public String type;
    }
    
    private FileName splitFileName(String fileName) {
        FileName fn = new FileName();
        fn.fullName = fileName;
        int dotPos = fileName.lastIndexOf('.');
        if (dotPos < 1)
            fn.type = "";
        else
            fn.type = fileName.substring(dotPos + 1);
        int linePos = -1;
        if (dotPos > 0) {
            String s = fileName.substring(0, dotPos);
            linePos = s.lastIndexOf('_');
            if (linePos < 0)
                fn.localeName = "";
            else {
                String localeName = s.substring(linePos + 1);
                if (localeName.equals("none"))
                    fn.localeName = localeName;
                else {
                    if (project.getLocale(localeName) != null)
                        fn.localeName = localeName;
                    else
                        fn.localeName = "";
                }
            }
        }
        if (linePos > 0)
            fn.name = fileName.substring(0, linePos);
        else if (dotPos > 0)
            fn.name = fileName.substring(0, dotPos);
        else
            fn.name = fileName;
        return fn;
    }
    
    private void processDirectory(File directory, String relativePath) throws ScException {
        System.out.println("Entering directory '" + directory.getPath() + "'");
        File files[] = directory.listFiles();
        for (int i = 0; i < files.length; i++) {
            File f = files[i];
            if (f.getName().startsWith("."))
                continue;
            if (f.isDirectory()) {
                String newPath;
                if (! relativePath.equals(""))
                  /*  newPath = relativePath + System.getProperties().getProperty("path.separator") 
                            + f.getName(); */
                    newPath = relativePath + '/' + f.getName();
                else
                    newPath = f.getName();
                processDirectory(f, newPath);
            } else {
                FileName fn = splitFileName(f.getName());
                if (((locale == null) && (! fn.localeName.equals("none"))) || ((locale != null) &&
                        fn.localeName.equals("none")))
                {
                  /*  System.out.println("Ignoring file " + f.getPath()); */
                    continue;
                } else {
                    if ((fn.localeName.length() > 0) && 
                            ((locale != null) && (! locale.getName().equals(fn.localeName))))
                        continue;
                }
                FileHandler handler = FileHandlersFactory.getHandler(fn.type);
                if (handler != null) {
                    System.out.println("Processing file " + f.getPath());
                    handler.processFile(f, new File(outputDir, relativePath), relativePath, 
                            fn.name, locale, fn.type, this);
                } /* else
                    System.out.println("Ignoring file " + f.getPath()); */
            }
        }
        System.out.println("Leaving directory '" + directory + "'");
    }
    
    public Project getProject() {
        return project;
    }
}
