/*
 * KeywordsRegistry.java
 *
 * Created on 17  2002 ., 12:15
 */

package com.fn.spcave.compiler;


import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 *
 * @author  asso
 */
public class KeywordsRegistry {

    private static Map classes;
    private static Map args;

    static {
        classes = new HashMap();
        args = new HashMap();
        try {
            addKeyword("set", "com.fn.spcave.compiler.SetKeyword", true);
            addKeyword("while", "com.fn.spcave.compiler.WhileKeyword", true);
            addKeyword("if", "com.fn.spcave.compiler.IfKeyword", true);
            addKeyword("else", "com.fn.spcave.compiler.ElseKeyword", false);
            addKeyword("unset", "com.fn.spcave.compiler.UnsetKeyword", true);
            addKeyword("clearBuffer", "com.fn.spcave.compiler.ClearBufferKeyword", false);
            addKeyword("dumpBuffer", "com.fn.spcave.compiler.DumpBufferKeyword", true);
        } catch (Exception e) { }
    }

    public static boolean isKeyword(String name) {
        return (classes.get(name) != null);
    }
    
    public static boolean hasArgumets(String name) {
        Boolean b = (Boolean)args.get(name);
        if (b == null) return false;
        return b.booleanValue();
    }
    
    public static void addKeyword(String keyword, String className, 
            boolean hasArguments) throws ScException 
    {
        try {
            Class cl = Class.forName(className);
            Class[] arguments = new Class[] { Lexical.class, String.class, int.class, 
                    int.class, List.class };
            Constructor cn = cl.getConstructor(arguments);
            if (cn != null)
                classes.put(keyword, cn);
            args.put(keyword, new Boolean(hasArguments));
        } catch (Exception e) {
            throw new ScException("Error adding keyword '" + keyword + ":" + className + "': "
                    + e.getMessage());
        }
    }

    public static KeywordInstruction createKeywordInstruction(String keyword,
                Lexical lexical, String source, int line, int position, 
                List arguments) throws ScException 
    {
        Constructor cn = (Constructor)classes.get(keyword);
        if (cn == null)
            return null;
        Object[] args = new Object[] { lexical, source, new Integer(line), 
            new Integer(position), arguments };
        try { 
            return (KeywordInstruction)cn.newInstance(args);
        } catch (IllegalAccessException ie) {
            throw new ScException(source + ":" + Integer.toString(line) + 
                    ": Error creating keyword instruction: permission denided");
        } catch (java.lang.reflect.InvocationTargetException te) {
            throw new ScException(source + ":" + Integer.toString(line) + 
                    ": Error creating keyword instruction: invocation target exception");
        } catch (InstantiationException e) {
            throw new ScException(source + ":" + Integer.toString(line) + 
                    ": Error creating keyword instruction");
        }
    }    
}
