/*
 * ReaderHalper.java
 *
 * Created on 8  2002 ., 15:30
 */

package com.fn.spcave.compiler;

/**
 *
 * @author  asso
 */
public class ReaderHelper {

    public static void skipSpaces(SourceReader reader) throws ScException {
        int ch = reader.getChar();
        while ((ch != -1) && Character.isWhitespace((char)ch))
            ch = reader.getChar();
        reader.ungetChar(ch);
    }
    
    private static boolean isSymbol(char ch) {
        char symbols[] = { '{', '}', '(', ')', '+', '-', '<', '>', '=', '*', '/', '\\', '.', ',', 
                ':', '!' };
        for (int i = 0; i < symbols.length; i++)
            if (symbols[i] == ch)
                return true;
        return false;
    }

    public static String readString(SourceReader reader) throws ScException {
        skipSpaces(reader);
        int ch = reader.getChar();
        if (ch == -1)
            return "";
        if ((ch != '\'') && (ch != '"'))
            throw new ScException("Line " + reader.getLineNumber() + ": Invalid string constant");
        int quoteChar = ch;
        ch = reader.getChar();
        StringBuffer sb = new StringBuffer();
        while ((ch != -1) && (ch != quoteChar)) {
            if (ch == '\\') {
                ch = reader.getChar();
                if (ch == -1)
                    throw new ScException("Line " + reader.getLineNumber() + ": Unexpected end of file");
                switch (ch) {
                    case 'n': ch = '\n'; break;
                    case 't': ch = '\t'; break;
                    case 'r': ch = '\r'; break;
                    case '\'': ch = '\''; break;
                    case '"': ch = '"'; break;
                    case '\\': ch = '\\'; break;
                }
            }
            sb.append((char)ch);
            ch = reader.getChar();
        }
        if (ch == quoteChar)
            return sb.toString();
        else
            throw new ScException("Line " + reader.getLineNumber() + ": String constant not terminated");
    }

    public static String readSymbol(SourceReader reader) throws ScException {
        skipSpaces(reader);
        int ch = reader.getChar();
        if (ch == -1)
            return "";
        if (! isSymbol((char)ch))
            throw new ScException("Line " + reader.getLineNumber() + ": Unexpected character '" + ch + "'");
        StringBuffer sb = new StringBuffer();
        sb.append((char)ch);
        if ((ch == '>') || (ch == '<')) {
            ch = reader.getChar();
            if (ch == '=')
                sb.append((char)ch);
            else
                reader.ungetChar(ch);
        }
        return sb.toString();
    }

    
    public static String readNumber(SourceReader reader) throws ScException {
        skipSpaces(reader);
        int ch = reader.getChar();
        if (ch == -1)
            return "";
        StringBuffer sb = new StringBuffer();
        while ((ch != -1) && (((ch >= '0') && (ch <= '9')) || (ch == '.'))) {
            sb.append((char)ch);
            ch = reader.getChar();
        }
        reader.ungetChar(ch);
        return sb.toString();
    }
    

    public static String readWord(SourceReader reader) throws ScException {
        skipSpaces(reader);
        int ch = reader.getChar();
        if (ch == -1)
            return "";
        if (! Character.isJavaIdentifierStart((char)ch))
            throw new ScException("Line " + reader.getLineNumber() + ": Invalid keyword start");
        StringBuffer sb = new StringBuffer();
        while ((ch != -1) && Character.isJavaIdentifierPart((char)ch)) {
            sb.append((char)ch);
            ch = reader.getChar();
        }
        reader.ungetChar(ch);
        return sb.toString();
    }
    
    
    public static String readToken(SourceReader reader) throws ScException {
        skipSpaces(reader);
        int ch = reader.getChar();
        reader.ungetChar(ch);
        if (ch == -1)
            return "";
        if ((ch == '"') || (ch == '\''))
            return readString(reader);
        if (isSymbol((char)ch))
            return readSymbol(reader);
        if ((ch >= '0') && (ch <= '9'))
            return readNumber(reader);
        if (((ch >= 'a') && (ch <= 'z')) || ((ch >= 'a') && (ch <= 'z')))
            return readWord(reader);
        throw new ScException("Line " + reader.getLineNumber() + 
                ": Token expected, but something else found");
    }


    public static String readTextBlock(SourceReader reader) throws ScException {
        skipSpaces(reader);
        int ch = reader.getChar();
        if (ch == -1)
            return "";
        if ((ch != '{'))
            throw new ScException("Line " + reader.getLineNumber() + ": Invalid text block");
        skipSpaces(reader);
        ch = reader.getChar();
        StringBuffer sb = new StringBuffer();
        while ((ch != -1) && (ch != '}')) {
            if (ch == '\\') {
                ch = reader.getChar();
                if (ch == -1)
                    throw new ScException("Line " + reader.getLineNumber() + ": Unexpected end of file");
                switch (ch) {
                    case 'n': ch = '\n'; break;
                    case 't': ch = '\t'; break;
                    case 'r': ch = '\r'; break;
                    case '\'': ch = '\''; break;
                    case '"': ch = '"'; break;
                    case '\\': ch = '\\'; break;
                    case '}': ch = '}'; break;
                    case '{': ch = '{'; break;
                }
            }
            sb.append((char)ch);
            ch = reader.getChar();
        }
        if (ch == '}')
            return sb.toString().trim();
        else
            throw new ScException("Line " + reader.getLineNumber() + ": Bock is not terminated");
    }
}
